/*
 * network.h
 * Network (wlan and eth) init and connect funtions for esp32
 *
 *  Created on: Feb 9, 2018
 *      Author: Armin Diehl <ad@ardiehl.de>
 */

#ifndef MAIN_NETWORK_H_
#define MAIN_NETWORK_H_

#include <stdio.h>
#include "sdkconfig.h"

#if defined (CONFIG_COMMANDS_WLAN) || defined (CONFIG_COMMANDS_ETH)
#define CONFIG_COMMANDS_NETWORK
#endif

#ifdef CONFIG_COMMANDS_NETWORK


#define TZ_DEFAULT "CET-1CEST,M3.5.0,M10.5.0/3"

// default connect timeout ms
#define DEFAULT_WIFI_CONNECT_TIMEOUT_MS		10000

#define DEFAULT_ETH_CONNECT_TIMEOUT_MS		10000

// time tp wait between tries to check/connect network
#define NETCON_RETRY_WAIT	10000

// number of NETCON_RETRY_WAIT for ntp reinitialization (should be within LWIP but does not seem to work)
#define SNTP_REINIT_WAIT	(6*60*2)	/* 2 hours */

void setTimezone();

// initialize (if needed) and try to connect to given WLAN connect*/
bool wifi_connect(FILE *fd, const char* ssid, const char* pass, int timeout_ms);

// connect to saved SSID using saved password
int network_connectWifi(FILE *fp);

void networkTask_start(FILE*fp);  // start a task that will try to connect to the network every x seconds

void initialize_sntp(void);

// save settings to configfile
void network_WifiSaveSettings (int wifiDefault, int wifiTimeoutMS, const char * ssid, const char *pwd);
void network_WifiSaveSettings (int wifiDefault);

#ifdef CONFIG_COMMANDS_ETH
bool eth_connect (FILE *out, int timeout_ms);

void network_EthSaveSettings (int ethDefault, int ethTimeoutMS);

int network_EthGetTimeout(void);
#endif

#endif /* CONFIG_COMMANDS_NETWORK */

int network_isConnected();


#endif /* MAIN_NETWORK_H_ */
