/*
 * update.cpp
 *
 *  Created on: Jan 25, 2018
 *      Author: ad
 *  mainly taken from OTA example code
 *
 *  June 24, 2018 AD: allow toggle of active partitions
 */


#include "update.h"

#ifdef ESP32
#include "esp_ota_ops.h"
#endif

#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <string.h>
#include <errno.h>
#include "cons.h"

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"


#define BUFFSIZE 16384
#define TEXT_BUFFSIZE 16384

#define SMALL_DELAY vTaskDelay(1);
#define MED_DELAY vTaskDelay(1);


/*an ota data write buffer ready to write to the flash*/
static char ota_write_data[BUFFSIZE + 1] = { 0 };
/*an packet receive buffer*/
static char text[BUFFSIZE + 1] = { 0 };
/* an image total length*/
static int binary_file_length;

#define YIELD() vTaskDelay(50)

int socketConnect (FILE *fd, const char *addrOrName, const char *port) {
	int sockfd,err;
	struct addrinfo hints;
    struct addrinfo *res;

    memset(&hints, 0, sizeof(hints));
    //hints.ai_family=AF_UNSPEC;
    hints.ai_family = AF_INET;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_protocol = IPPROTO_TCP;

	err = getaddrinfo(addrOrName, port, &hints, &res);

    if (err != 0 || res == NULL) {
        fprintf(fd,"socketConnect: getaddrinfo for '%s'failed err=%d res=%p\r\n", addrOrName, err, res);
        return -1;
    }

    sockfd = socket(res->ai_family, res->ai_socktype, 0);
    YIELD();
    if (sockfd < 0) {
       fprintf(fd,"socketConnect: Failed to allocate socket\r\n");
       freeaddrinfo(res);
       return -1;
    }
    if (connect(sockfd, res->ai_addr, res->ai_addrlen) != 0) {
       fprintf(fd,"socketConnect: socket connect failed errno=%d\r\n", errno);
       close(sockfd);
       freeaddrinfo(res);
       return -1;
    }

    freeaddrinfo(res);
    return sockfd;
}

/*read buffer by byte still delim ,return read bytes counts*/
static int read_until(char *buffer, char delim, int len)
{
//  /*TODO: delim check,buffer check,further: do an buffer length limited*/
    int i = 0;
    while (buffer[i] != delim && i < len) {
        ++i;
    }
    return i + 1;
}

/* resolve a packet from http socket
 * return true if packet including \r\n\r\n that means http packet header finished,start to receive packet body
 * otherwise return false
 * */
static bool read_past_http_header(FILE *fd, char text[], int total_len, esp_ota_handle_t update_handle, int testOnly)
{
	esp_err_t err;

    /* i means current position */
    int i = 0, i_read_len = 0;
    while (text[i] != 0 && i < total_len) {
        i_read_len = read_until(&text[i], '\n', total_len);
        // if we resolve \r\n line,we think packet header is finished
        if (i_read_len == 2) {
            int i_write_len = total_len - (i + 2);
            memset(ota_write_data, 0, BUFFSIZE);
            /*copy first http packet body to write buffer*/
            memcpy(ota_write_data, &(text[i + 2]), i_write_len);

            if (testOnly) {
            	err = ESP_OK;
            } else
            	err = esp_ota_write( update_handle, (const void *)ota_write_data, i_write_len);
            if (err != ESP_OK) {
                fprintf(fd, "Error: esp_ota_write failed! err=0x%x", err);
                return false;
            } else {
                //ESP_LOGI(TAG, "esp_ota_write header OK");
                binary_file_length += i_write_len;
            }
            return true;
        }
        i += i_read_len;
    }
    return false;
}


#ifdef ESP32
static void printPartInfo(FILE *fd)
{
    const esp_partition_t *configured = esp_ota_get_boot_partition();
    const esp_partition_t *running = esp_ota_get_running_partition();

    if (configured != running) {
        fprintf(fd, "Configured OTA boot partition at offset 0x%08x, but running from offset 0x%08x. (This can happen if either the OTA boot data or preferred boot image become corrupted somehow.)\r\n",
        		configured->address, running->address);
    }
    fprintf(fd, "Current active partition type %d subtype %d (offset 0x%08x)\r\n", running->type, running->subtype, running->address); fflush(fd);
}
#endif


#ifdef ESP32
int toggleActivePartition (FILE *fd) {
	esp_err_t err;
	esp_partition_t * new_partition;

	printPartInfo(fd);

	new_partition = (esp_partition_t *)esp_ota_get_next_update_partition(NULL);

    if (new_partition == NULL) {
    	fprintf(fd, CONS_COLOR_STR_E_N("update_partition is NULL ???"));
    	return 1;
    }
    fprintf(fd, "new active partion will be subtype %d at offset 0x%x\r\n", new_partition->subtype, new_partition->address);

	err = esp_ota_set_boot_partition(new_partition);
    if (err != ESP_OK) {
    	fprintf(fd, CONS_COLOR_STR_E_N("esp_ota_set_boot_partition failed! err=0x%x"), err);
    	return 1;
    }
    fprintf(fd,"partition has been changed.\r\n");
    return 0;
}
#else
int toggleActivePartition (FILE *fd) {
	fprintf(fd,"ESP32 would change active partition.\r\n");
	return 1;
}
#endif

int update_firmware (FILE *fd, char * server, char * path, int port, int testOnly) {
	int sockfd;
	char portS[6];
	esp_err_t err;
	esp_ota_handle_t update_handle = 0;
	int packetCount = 0;
	TickType_t xTimeBefore, xTotalTime;

	fprintf(fd,CONS_COLOR_STR_I_N("trying to update firmware from http://%s:%s:%d"),server,path,port);
	if (testOnly) fprintf(fd,CONS_COLOR_STR_E_N("TEST ONLY, NO FLASH WRITE WILL BE PERFORMED"));
#ifdef ESP32
	printPartInfo(fd);
#endif
	xTimeBefore = xTaskGetTickCount();
	binary_file_length = 0;
	sprintf(portS,"%d",port);
	sockfd = socketConnect (fd,server,&portS[0]);
	if (!sockfd) {
		fprintf(fd,CONS_COLOR_STR_E_N("unable to connect to %s:%d"),server,port);
		return 0;
	}

	    /*send GET request to http server*/
    const char *GET_FORMAT =
        "GET %s HTTP/1.0\r\n"
        "Host: %s:%s\r\n"
        "User-Agent: esp-idf/1.0 esp32\r\n\r\n";

    char *http_request = NULL;
    int get_len = asprintf(&http_request, GET_FORMAT, path, server, portS);
    if (get_len < 0) {
        fprintf(fd, CONS_COLOR_STR_E_N("Failed to allocate memory for GET request buffer"));
        close(sockfd);
        return false;
    }
    int res = send(sockfd, http_request, get_len, 0);
    free(http_request);

    if (res < 0) {
        fprintf(fd, CONS_COLOR_STR_E_N("Send GET request to server failed"));
        close(sockfd);
        return false;
    } else {
        //ESP_LOGI(TAG, "Send GET request to server succeeded");
    }

    esp_partition_t * update_partition = (esp_partition_t *)esp_ota_get_next_update_partition(NULL);
    if (update_partition == NULL) {
    	fprintf(fd, CONS_COLOR_STR_E_N("update_partition is NULL ???"));
    	close(sockfd);
    	return false;
    }
    fprintf(fd, "Writing to partition subtype %d at offset 0x%x\r\n", update_partition->subtype, update_partition->address);

    if (testOnly) {
    	err = 0;
    } else {
    	err = esp_ota_begin(update_partition, OTA_SIZE_UNKNOWN, &update_handle);
    }
    if (err != ESP_OK) {
        fprintf(fd, CONS_COLOR_STR_E_N("esp_ota_begin failed, error=%d"), err);
        close(sockfd);
        return false;
    }
    fprintf(fd, "esp_ota_begin succeeded\r\n");
    fflush(fd);
    MED_DELAY;

    bool resp_body_start = false, flag = true;
    /*deal with all receive packet*/
    while (flag) {
    	SMALL_DELAY;
        memset(text, 0, TEXT_BUFFSIZE);
        memset(ota_write_data, 0, BUFFSIZE);
        int buff_len = recv(sockfd, text, TEXT_BUFFSIZE, 0);
        if (buff_len < 0) { /*receive error*/
            fprintf(fd, CONS_COLOR_STR_E_N("\r\nError: receive data error! errno=%d"), errno);
            close(sockfd);
            return false;
        } else if (buff_len > 0 && !resp_body_start) { /*deal with response header*/
            memcpy(ota_write_data, text, buff_len);
            resp_body_start = read_past_http_header(fd, text, buff_len, update_handle, testOnly);
        } else if (buff_len > 0 && resp_body_start) { /*deal with response body*/
            memcpy(ota_write_data, text, buff_len);
            if (testOnly) {
            	err = 0;
            } else {
            	err = esp_ota_write( update_handle, (const void *)ota_write_data, buff_len);
            }
            SMALL_DELAY;
            if (err != ESP_OK) {
                fprintf(fd, CONS_COLOR_STR_E_N("\r\nError: esp_ota_write failed! err=0x%x"), err);
                close(sockfd);
                return false;
            }
            binary_file_length += buff_len;
            packetCount++;
            if ((packetCount / 8 * 8) == packetCount)
            	fprintf(fd, "\r%d  ", binary_file_length); fflush(fd);
        } else if (buff_len == 0) {  /*packet over*/
            flag = false;
            fprintf(fd,"\r\nConnection closed, all packets received\r\n");
            close(sockfd);
        } else {
            fprintf(fd, CONS_COLOR_STR_E_N("Error: Unexpected recv result"));
        }
    }

    fprintf(fd,"\r\nTotal Write binary data length : %d\r\n", binary_file_length);

    if (testOnly) {
    	err = 0;
    } else {
    	err = esp_ota_end(update_handle);
    }
    if (err != ESP_OK) {
        fprintf(fd, CONS_COLOR_STR_E_N("esp_ota_end failed!"));
        return false;
    }
#ifdef ESP32
    if (!testOnly) {
    	err = esp_ota_set_boot_partition(update_partition);
    	if (err != ESP_OK) {
    		fprintf(fd, CONS_COLOR_STR_E_N("esp_ota_set_boot_partition failed! err=0x%x"), err);
    		return false;
    	}
    }

#endif
    xTotalTime = (xTaskGetTickCount() - xTimeBefore) * portTICK_PERIOD_MS;
    fprintf(fd,"update took %d seconds\r\n",xTotalTime / 1000);
    fflush(fd);
    vTaskDelay(pdMS_TO_TICKS(250));

    return true;
}
