/*
 * configfile.h: very simple ini file like (esp32 and linux)
 *
 *  Created on: Jan 2, 2018
 *      Author: Armin Diehl <ad@ardiehl.de>
 */

#ifndef COMPONENTS_CONFIGFILE_H_
#define COMPONENTS_CONFIGFILE_H_

#include <stdint.h>


#define CONFIGFILE_MAX_LINES 100
#define CONFIGFILE_LINE_LEN  128



#define CONFIGFILE_ERR_OK     0
#define CONFIGFILE_ERR_TOLONG 1
#define CONFIGFILE_ERR_FULL   2
#define CONFIGFILE_ERR_FILE_NOT_FOUND   3
#define CONFIGFILE_ERR_FILE_CREATE      4
#define CONFIGFILE_ERR_NOT_FOUND        5
#define CONFIGFILE_ERR_NOMEM            6

class configfile_t
{
  public:
	configfile_t();	// read the given filename if not NULL
	//~configfile_t();						// save changes at exit

	int set_u32 (const char *name, uint32_t value);
	int set_str (const char *name, const char *value);

	int get_u32 (const char *name);				// 0 if not set
	char * get_str (const char *name);			// NULL if not defined, returned pointer needs to be free'ed

	int load (const char * configFilename);
	int save (const char * configFilename);
	int save();							// save if something was changed

	char * filename();					// return current file name, do not touch or free returned string


  private:
	int nLines;
	char *lines[CONFIGFILE_MAX_LINES];
	char *lastFilename;
	int changed;

	int add(const char *c);
	int add(const char *name, const char * value);
	char * findName(const char *name);
	int findNameIdx(const char *name);
	void freeConfigData();

};


extern configfile_t configfile;


#endif /* COMPONENTS_CONFIGFILE_H_ */
