/*
 * globalLock.h
 *
 *  Created on: Jun 12, 2018
 *      Author: Armin Diehl <ad@ardiehl.de>
 */


//#include <stdio.h>
//#include <string.h>

#include "globalLock.h"
#include "esp_log.h"
#include "freertos/FreeRTOS.h"
#include "freertos/semphr.h"

static const char *TAG = "globalLock";

SemaphoreHandle_t gsem[GLOCK_MAX+1];

void globalLockInit (int type) {
    if (type > GLOCK_MAX) return;
    if (gsem[type] == 0) gsem[type] = xSemaphoreCreateMutex();
}

void globalLockTake (int type) {
    BaseType_t res;

    if (type > GLOCK_MAX) return;
    if (gsem[type] == 0) gsem[type] = xSemaphoreCreateMutex();
    do {
    	res = xSemaphoreTake(gsem[type],10000 / portTICK_PERIOD_MS);  // 10 seconds timeout
    	if (res != pdTRUE) ESP_LOGE(TAG,"unable to take lock for %d, possible deadlock, continuing wait, res: %d",type,(int)res);
    } while (res != pdTRUE);
}

void globalLockGive (int type) {
    if (type > GLOCK_MAX) return;
    xSemaphoreGive(gsem[type]);
}



