/*
 * evse_portio.cpp
 *
 *  Created on: Jun 15, 2018
 *      Author: ad
 */


#include <string.h>
#include "globalLock.h"
#include "esp_log.h"
#include "mcp23017.h"
#include "mcp23017_portio.h"


mcp23017_t mcp[PIO_MAX];
static const char* TAG = "evse_portio";

// ---- configuration ---------------------------------------------------------------------

#define LOCK(mcpIdx)   globalLockTake ( (mcp[mcpIdx].port==I2C_NUM_0) ? GLOCK_I2C_0 : GLOCK_I2C_1)
#define UNLOCK(mcpIdx) globalLockGive ( (mcp[mcpIdx].port==I2C_NUM_0) ? GLOCK_I2C_0 : GLOCK_I2C_1)


int pio_init(int mcpIdx, uint8_t i2c_addr, i2c_port_t port) {
#ifdef ESP_PLATFORM
	mcp23017_err_t err;

	if (mcpIdx > PIO_MAX) return 1;

	memset(&mcp[mcpIdx], 0, sizeof(mcp[0]));

	mcp[mcpIdx].i2c_addr = i2c_addr;
	mcp[mcpIdx].port = port;

	LOCK(mcpIdx);

	err = mcp23017_init(&mcp[mcpIdx], 0);

	UNLOCK(mcpIdx);

	if (err != MCP23017_ERR_OK) {
		ESP_LOGE(TAG,"mcp23017_init failed with %d, idx:%d, addr: %d",err,mcpIdx,i2c_addr);
		return 1;
	}
#else
	ESP_LOGD(TAG,"pio_init (%d, %d, %d)",mcpIdx, i2c_addr, port);
#endif
	return 0;
}

#ifdef ESP_PLATFORM
int pio_init(int mcpIdx, mcp23017_t * nmcp) {
	if (mcpIdx > PIO_MAX) return 1;

	mcp23017_err_t err;

	mcp[mcpIdx] = * nmcp;
	LOCK(mcpIdx);
	err = mcp23017_init(&mcp[mcpIdx], 1);
	UNLOCK(mcpIdx);
	if (err != MCP23017_ERR_OK) {
		ESP_LOGE(TAG,"mcp23017_init failed with %d, idx:%d, addr: %d",err,mcpIdx,nmcp->i2c_addr);
		return 1;
	}


	return 0;
}
#endif


int chReg (int mcpIdx, mcp23017_reg_t reg, mcp23017_gpio_t group, uint8_t mask, int setBits) {
	uint8_t regVal,prevVal;
	mcp23017_err_t err;

	LOCK(mcpIdx);
	err = mcp23017_read_register(&mcp[mcpIdx], reg, group, &regVal);
	if (err != MCP23017_ERR_OK) {
		UNLOCK(mcpIdx);
		ESP_LOGE(TAG,"chReg, read failed with %d, idx:%d",err,mcpIdx);
		return 1;
	} else {
		prevVal = regVal;
		if (setBits) {
			regVal |= mask;
		} else {
			regVal &= ~mask;
		}
		err = mcp23017_write_register(&mcp[mcpIdx], reg, group, regVal);
		UNLOCK(mcpIdx);
		if (err != MCP23017_ERR_OK) {
			ESP_LOGE(TAG,"chReg, write failed with %d, idx:%d",err,mcpIdx);
			return 1;
		} else {
			ESP_LOGD(TAG,"chReg (%d, %d, %d, 0x%02x), newVal: 0x%02x, prevVal: 0x%02x",mcpIdx,reg,group,mask,regVal,prevVal);
		}
	}
	return 0;
}

// set pins given (=1) to output. leave other pin configuration untouched
// Register: IODIR: 1=Input, 0=output
int pio_setPinsOutput (int mcpIdx, uint16_t mask) {
	int res, err;

	res = 0;
	err = chReg (mcpIdx, MCP23017_IODIR, GPIOA, mask, false);
	if (err != 0) res++;
	err = chReg (mcpIdx, MCP23017_IODIR, GPIOB, (mask >> 8) & 0xff, false);
	if (err != 0) res++;
	return res;
}

// set pins given (=1) to input. leave other pin configuration untouched
int pio_setPinsInput (int mcpIdx, uint16_t mask, int pullup) {
	int res, err;

	res = 0;
	err = chReg (mcpIdx, MCP23017_IODIR, GPIOA, mask, true);
	if (err != 0) res++;
	err = chReg (mcpIdx, MCP23017_GPPU, GPIOA, mask, pullup);
	if (err != 0) res++;

	err = chReg (mcpIdx, MCP23017_IODIR, GPIOB, (mask >> 8) & 0xff, true);
	if (err != 0) res++;
	err = chReg (mcpIdx, MCP23017_GPPU, GPIOB, mask, pullup);
	if (err != 0) res++;
	return res;
}


// ---- in/out ----------------------------------------------------------------------------
int pio_getByte (int mcpIdx, int hiByte, uint8_t * value) {
	mcp23017_err_t err;
	mcp23017_gpio_t g = GPIOA;

	if (hiByte) g = GPIOB;
	LOCK(mcpIdx);
	err = mcp23017_read_register(&mcp[mcpIdx], MCP23017_GPIO, g, value);
	UNLOCK(mcpIdx);
	return (int)err;
}


int pio_setPins (int mcpIdx, uint16_t mask) {
	int ret = 0;

	if (mask & 0xff)
		ret = chReg (mcpIdx, MCP23017_GPIO, GPIOA, mask, true);
	if (mask & 0xff00)
		ret += chReg (mcpIdx, MCP23017_GPIO, GPIOB, (mask >> 8) & 0xff, true);

	return ret;
}


int pio_clearPins (int mcpIdx, uint16_t mask) {
	int ret = 0;

	if (mask & 0xff)
		ret = chReg (mcpIdx, MCP23017_GPIO, GPIOA, mask, false);
	if (mask & 0xff00)
		ret += chReg (mcpIdx, MCP23017_GPIO, GPIOB, (mask >> 8) & 0xff, false);

	return ret;
}

