/* EVSE_AD - energyMeter
   Handles reading the SDM630 modbus energy meter

   Armin Diehl <ad@ardiehl.de> December 2017

   Unless required by applicable law or agreed to in writing, this
   software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
   CONDITIONS OF ANY KIND, either express or implied.
*/


#ifndef ENERGYMETER_H
#define ENERGYMETER_H

#ifdef ESP32
#include "driver/gpio.h"
#include "driver/uart.h"

#define EM_UART_NUM UART_NUM_1
#define EM_TXD_PIN (GPIO_NUM_12)
// if GPIO14 is connected, board no longer boots (RTCWDT_RTC_RESET)
#define EM_RXD_PIN (GPIO_NUM_14)
#define EM_BAUD 9600

#else

#include <termios.h>

#define EM_UART_NUM 1
#define EM_UART_PORT "/dev/ttyUSB1"
#define EM_BAUD B9600
#endif

//#define EM_RXD_PIN (GPIO_NUM_5)

#define EM_TXD_BUFSIZE 0
#define EM_RXD_BUFSIZE 256
#define EM_QUERY_EVERY_MSECS 1000
// query KWH only on every 30th V/A/P query
#define EM_QUERY_KWH_INTERVAL 30
#define EM_QUERY_ERRPAUSE_MSECS 5000

struct em_data_t {
	int		pwrL1;
	int 	pwrL2;
	int		pwrL3;
	float	currL1;
	float 	currL2;
	float	currL3;
	float	voltageL1;
	float	voltageL2;
	float	voltageL3;
	int		kwh;
	int		kwhL1;
	int		kwhL2;
	int		kwhL3;
};

int queryEnergyMeter_startTask(FILE *fp);
int queryEnergyMeter_endTask(FILE *fd);

int energyMeter_getPower (int * numPhases, int * powerConsumption);
int energyMeter_getPower3 (	int * powerConsumptionL1,
							int * powerConsumptionL2,
							int * powerConsumptionL3);

int energyMeter_getCurrent3(int * currentL1,
							int * currentL2,
							int * currentL3);

em_data_t * energyMeter_getCurrentData();

void register_energyMeterCommands();  /* register console commands */

#endif
