/*
 * evse_event.cpp
 *
 *  Created on: Dec 19, 2017
 *      Author: ad
 */

#include "evse_global.h"

static const char* TAG = "evse_event";

// queue handles for each task
QueueHandle_t eventQueues[EV_TGT_TASKS];


const char * eventTxt(ev_event_t e) {

	switch (e) {
		case EV_NET_CONNECTED		: return "EV_NET_CONNECTED"; break;
		case EV_NET_DISCONNECTED 	: return "EV_NET_DISCONNECTED"; break;
		case EV_KEY_PRESS			: return "EV_KEY_PRESS"; break;
		case EV_KEY_RELEASE			: return "EV_KEY_RELEASE"; break;
		case EV_UI_TIMER_PAGE		: return "EV_UI_TIMER_PAGE"; break;
		case EV_UI_TIMER_TIMEOUT	: return "EV_UI_TIMER_TIMEOUT"; break;
		case EV_UI_REDRAW			: return "EV_UI_REDRAW"; break;
		case EV_AVAILABLE_PWR_CHANGE: return "EV_AVAILABLE_PWR_CHANGE"; break;
		case EV_CONSUMED_PWR_CHANGE : return "EV_CONSUMED_PWR_CHANGE"; break;
		case EV_CABLE_CHANGE		: return "EV_CABLE_CHANGE"; break;
		case EV_CAR_CHARGE_VENT		: return "EV_CAR_CHARGE_VENT"; break;
		case EV_CAR_CHARGE			: return "EV_CAR_CHARGE"; break;
		case EV_CAR_READY			: return "EV_CAR_READY"; break;
		case EV_END_TASK			: return "EV_END_TASK"; break;
		default:
			return "EV_UNKNOWN";
	}
}

void ev_initialize(void) {
	for (int i=0; i<EV_TGT_TASKS; i++) {
		eventQueues[i] = xQueueCreate(EV_QUEUE_LENGTH,sizeof(ev_event_data_t));
		if (eventQueues[i] == NULL) { ESP_LOGE(TAG, "xQueueCreate failed %d",i); }
		// else ESP_LOGD(TAG, "created queue %d",i);
	}
	// queue a redraw event for the UI to make sure the first page is visible
	ev_send (EV_TASK_UI, EV_UI_REDRAW, 0);
}

void ev_send (int targets, ev_event_t event, ev_data_t data) {
	int t = 1;
	ev_event_data_t e;

	e.event = event;
	e.data = data;

	for (int i=0; i<EV_TGT_TASKS; i++) {
		if (targets & t) {
			//ESP_LOGD(TAG,"ev_send (%d,%d,%d) q:%d",targets,event,data,i);
			xQueueSend(eventQueues[i],&e,0);
		}
		t = t << 1;
	}
}


int ev_receive (int whoAmI, ev_event_t *event, ev_data_t *data, TickType_t xTicksToWait) {
	int t = 1;
	ev_event_data_t e;

	for (int i=0; i<EV_TGT_TASKS; i++) {
		if (whoAmI & t) {
			if (xQueueReceive(eventQueues[i],&e,xTicksToWait)) {
				*data = e.data;
				*event = e.event;
				//ESP_LOGI(TAG,"ev_receive(%04x) returns event %d",whoAmI,*event);
				return EV_RC_OK;
			} else return EV_RC_NOEVENT;
		}
		t = t << 1;
	}
	//ESP_LOGI(TAG,"ev_receive(%04x) no event",whoAmI);
	return EV_RC_INVALID;
}




