/* EVSE_AD
   Car charger controlled by available solar energy querying a fronius inverter

   Armin Diehl <ad@ardiehl.de> December 2017

   Unless required by applicable law or agreed to in writing, this
   software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
   CONDITIONS OF ANY KIND, either express or implied.
*/

#include "evse_global.h"

#ifdef ESP_PLATFORM
#include "driver/ledc.h"
#include "esp_vfs_dev.h"
#include "driver/uart.h"

#include "esp_vfs_fat.h"
#include "nvs_flash.h"
#include "nvs.h"

#include "tcpip_adapter.h"
#include "esp_wifi.h"
#include "esp_event_loop.h"
#include "eth_phy/phy_lan8720.h"
#include "mcp23017.h"

#include "evse_keys.h"

#else // ESP_PLATFORM
#include "esp_compat.h"
#endif // ESP_PLATFORM


#include "j1772pilot.h"
#include "energyMeter.h"
#include "display.h"

#include "evse_commands.h"
#include "energyMeter.h"
#include "queryInverter.h"

#include "network.h"
#include "evse_mqtt.h"
#include "mcp23017_portio.h"

#include "commands.h"
#include "version.h"

static const char* TAG = "evse_main";

#define CONFIG_FILENAME MOUNT_PATH "/evse.cfg"

#define CONFIG_STORE_HISTORY

/* Console command history can be stored to and loaded from a file.
 * The easiest way to do this is to use FATFS filesystem on top of
 * wear_levelling library.
 */
#ifdef CONFIG_STORE_HISTORY
#define HISTORY_PATH MOUNT_PATH "/history.txt"
#endif


//*** Callbacks for commands *************************************************************

extern "C" void networkConnectEvent(int connected) {
  if (connected) ev_send (EV_TASK_NET,EV_NET_CONNECTED,EV_DATA_NONE);
  else ev_send (EV_TASK_NET,EV_NET_DISCONNECTED,EV_DATA_NONE);
}

static const char *version_str_l = EVSE_VERSION_STR_L;

extern "C" char* ver_getVersionStrLong() {
	return (char *) &version_str_l[0];
}

static const char *version_str = EVSE_VERSION;

extern "C" char* ver_getVersionStr() {
	return (char *)&version_str[0];
}

static const char *version_author = EVSE_AUTOR;

extern "C" char* ver_getAuthor() {
	return (char *)&version_author[0];
}

//****************************************************************************************


void keytest() {
	/*
	int rc;
	char c;

	rc = read(fileno(stdin),&c,1);
	printf("read returned %d, c=%d\r\n",rc,c);
	*/
}



void test() {
/*
	vTaskDelay(2000 / portTICK_PERIOD_MS);
	esp_log_level_set(NULL,ESP_LOG_DEBUG);
	printf("all input\n");
	pio_setPinsInput  (0, 0xffff);
	printf("\n15, 9 and 0=out\n");
	pio_setPinsOutput (0, 0x8101);
	printf("\n15 and 0=in\n");
	pio_setPinsInput  (0, 0x8001);
	printf("\nall out\n");
	pio_setPinsOutput  (0, 0xffff);

	printf("\nset Pins 0x0312\n");
	pio_setPins (0, 0x0312);

	printf("\nclear Pins 0x0102\n");
	pio_clearPins (0, 0x0102);
*/
}





void initialize_loglevel(void) {
	int level = configfile.get_u32("LOGLEVEL");
	if (level < 1000) return;
#ifdef ESP_PLATFORM
	esp_log_level_set("*",esp_log_level_t(level-1000));
	//esp_log_level_set("*",esp_log_level_t(level-1000));
#endif
}


#ifdef ESP_PLATFORM
#define PR(S) printf("\r\n" S)
#else
#define PR(S) printf("\r\n " S); fflush(stdout)
#endif

#ifdef ESP_PLATFORM
extern "C" void app_main() {
#else
extern "C" void app_main(int argc, char* argv[]) {
#endif

	commands_Initialize();			// init console, register commands, init i2 and mcp23017
	j1772pwm_portsInitialize();		// avoid power relais switching on during restart

	printf(TAG); PR(" initializing \n");

	//keytest();

	PR("[event queues] ");	ev_initialize();                        // initialize event system for sending events to running evse tasks
	PR("[filesystem] ");	initialize_filesystem();
	PR("[configfile]");     configfile.load (CONFIG_FILENAME);
							initialize_loglevel();

#ifdef ESP_PLATFORM
	PR("[nvs_flash] ");		nvs_flash_init();
#endif
																	// read config file from filesystem in flash

    PR("[start net] ");     networkTask_start(stdout);         		// start eth or wifi based on configuration
    PR("[j1772] ");			j1772pwm_initialize(stdout);			// initialize the LED controller for PWM and start j1772 task
	PR("[display] ");		display_setup();						// setup display and start UI task
	PR("[enery meter] ");	queryEnergyMeter_startTask(stdout);		// start the modbus query task for the energy meter
	PR("[inverter] ");		queryInverter_initialize();				// start task to query the inverter for available power
#ifdef ESP_PLATFORM
	PR("[keyboard] ");		keys_initialize(stdout);				// start keyboard reader task
#endif
	register_keysCommands();
	register_energyMeterCommands();
	register_j1772commands();
	register_InvCommands();
	
	evseCommands_initialize(stdout);// register_evseCommands();

	PR("[start mqtt] ");    register_mqttCommands(); mqtt_startTask(stdout);

	PR("\n\n");

	showWelcomeMsg(stdout);

	test();

	// if running under linux FreeRTOS, execute all commands specifies on command line
#ifndef ESP_PLATFORM
	int i, ret;
	for (i=1;i<argc;i++) {
		printf(">%s\n",argv[i]);
		console_run(stdout, stdin, argv[i], &ret);
	}
#endif

    /* Main loop for console (serial on esp) */
    while (1) {
    	commandLoop(stdin,stdout);
#ifndef ESP_PLATFORM
    	return;
#endif
    }
}
