/* EVSE_AD - mqtt support
   Handles sending of various variables to an mqtt server (i use thingsboard)

   Armin Diehl <ad@ardiehl.de> Mar 10, 2018

   Jun 9, 2018 AD:	first working version

   Unless required by applicable law or agreed to in writing, this
   software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
   CONDITIONS OF ANY KIND, either express or implied.
*/


#include "evse_global.h"
#include "evse_mqtt.h"
#include "cons.h"
#include "argtable3_ad.h"
#include <stdarg.h>
#include "mqtt_client.h"
#include "version.h"
#include "energyMeter.h"
#include <string.h>

static const char* TAG = "mqtt";


//static int ignoreErrCount;

static int debug = 0;
static FILE * debug_stderr = NULL;
static const char * MODULE = "mqtt  ";

static void DMSG(
        const char* tag,
        const char* format, ...) __attribute__ ((format (printf, 2, 3)));

static void DMSG(
        const char* tag,
        const char* format, ...) {
	if (debug) {
		if (debug_stderr != NULL) {
			va_list list;
			va_start(list, format);
			fprintf(debug_stderr,CONS_COLOR_STR_I("\r%s"),MODULE);
			fprintf(debug_stderr,CONS_COLOR_STR_T("%-20s"), tag);
			vfprintf(debug_stderr,format, list);
			va_end(list);
			fprintf(debug_stderr,"\r\n");
			fflush(debug_stderr);
		}
		if (ferror(debug_stderr) != 0) {
			fclose(debug_stderr);
			debug_stderr = NULL;
			debug = 0;
			ESP_LOGE(TAG,"debug stdout no longer valid, debug mode disabled");
		}
	}
}




extern "C" esp_err_t mqtt_event_handler(esp_mqtt_event_handle_t event)
{
	const char * ME = __func__;
    //esp_mqtt_client_handle_t client = event->client;
    //int msg_id;
    // your_context_t *context = event->context;
    switch (event->event_id) {
        case MQTT_EVENT_CONNECTED:
            DMSG(ME,"MQTT_EVENT_CONNECTED");

            ev_send (EV_TASK_UI + EV_TASK_MQTT, EV_MQTT_CONNECTED, EV_DATA_NONE);

            //sprintf(&fw_ver[0],"{\"firmware_version\":\"%s\"}",EVSE_VERSION);

            //msg_id = esp_mqtt_client_publish(client, "v1/devices/me/attributes", (const char*)&fw_ver, strlen(fw_ver), 1, 1);
            //DMSG(ME, "sent publish successful, msg_id=%d", msg_id);

            //msg_id = esp_mqtt_client_subscribe(client, "/topic/qos0", 0);
            //DMSG(ME, "sent subscribe successful, msg_id=%d", msg_id);
/*
            msg_id = esp_mqtt_client_subscribe(client, "/topic/qos1", 1);
            DMSG(ME, "sent subscribe successful, msg_id=%d", msg_id);

            msg_id = esp_mqtt_client_unsubscribe(client, "/topic/qos1");
            DMSG(ME, "sent unsubscribe successful, msg_id=%d", msg_id);
*/
            break;
        case MQTT_EVENT_DISCONNECTED:
            DMSG(ME, "MQTT_EVENT_DISCONNECTED");
            ev_send (EV_TASK_UI || EV_TASK_MQTT, EV_MQTT_DISCONNECTED, EV_DATA_NONE);
            break;

        case MQTT_EVENT_SUBSCRIBED:
            DMSG(ME, "MQTT_EVENT_SUBSCRIBED, msg_id=%d", event->msg_id);
            //msg_id = esp_mqtt_client_publish(client, "/topic/qos0", "data", 0, 0, 0);
            //DMSG(ME, "sent publish successful, msg_id=%d", msg_id);
            break;
        case MQTT_EVENT_UNSUBSCRIBED:
            DMSG(ME, "MQTT_EVENT_UNSUBSCRIBED, msg_id=%d", event->msg_id);
            break;
        case MQTT_EVENT_PUBLISHED:
            DMSG(ME, "MQTT_EVENT_PUBLISHED");
            break;
        case MQTT_EVENT_DATA:
            DMSG(ME, "MQTT_EVENT_DATA  TOPIC=%.*s DATA=%.*s",
            		event->topic_len, event->topic,
					event->data_len, event->data);
            //printf("TOPIC=%.*s\r\n", event->topic_len, event->topic);
            //printf("DATA=%.*s\r\n", event->data_len, event->data);
            break;
        case MQTT_EVENT_ERROR:
            DMSG(ME, "MQTT_EVENT_ERROR");
            break;
    }
    return ESP_OK;
}



void mqtt_timer_sendData_callback (TimerHandle_t xTimer) {
	//ESP_LOGD(TAG,"ui_timer_callback");
	ev_send  (EV_TASK_MQTT, EV_MQTT_SEND_TIMER, EV_DATA_NONE);	// send the timer event
	xTimerStart(xTimer,100);									// and restart the timer
}

// some macros to make it more easy to generate the JSON string for publish
#define MQV_S "{"
#define MQV_N   ","
#define MQV_E   "}"
#define MQV_VAR(NAME,VALUE) "\"" NAME "\":\"" VALUE "\""
#define MQV_VARS(NAME) "\"" NAME "\":\"%s\""
#define MQV_VARD(NAME) "\"" NAME "\":\"%d\""
#define MQV_VARF(NAME) "\"" NAME "\":\"%03.1f\""

TaskHandle_t pvMqttTaskHandle = NULL;


esp_mqtt_client_config_t mqtt_cfg;
char fw_ver[120];
char varBuf[300];


extern "C" void mqttTask(void * UNUSED(param)) {
	const char * ME = __func__;
	int res,i;
	int sleepTime = MQ_SEND_EVERY_MSECS;
	int terminated = 0;
	ev_event_t event;
    ev_data_t data;
    char *tempStr;
    int connected;
    int msg_id;
    TimerHandle_t sendTimer;
    int v_powerFeedIntoGrid = 0;
    int v_phasesState = 0;
    int v_pwmAmps = 0;
    int v_cableAmps = 0;
    em_data_t *energyMeterData;
    int mqtt_sendInterval;
	
	ESP_LOGI(TAG,"mqtt task started");
	energyMeterData = energyMeter_getCurrentData();
	connected = 0;
	snprintf(&fw_ver[0],sizeof(fw_ver),
			MQV_S \
			  MQV_VARS("firmware_version")    MQV_N \
			  MQV_VARS("firmware_date")       MQV_N \
			  MQV_VARS("firmware_compiled") \
			MQV_E,
			EVSE_VERSION,EVSE_VERSION_DATE,EVSE_COMPILE_DATE);

	// interval
	mqtt_sendInterval = configfile.get_u32(MQ_CFG_INTERVAL) * 1000;
	if (mqtt_sendInterval < MQ_SEND_EVERY_MSECS_MIN) mqtt_sendInterval = MQ_SEND_EVERY_MSECS_MIN;

    // server
	tempStr = configfile.get_str (MQ_CFG_SERVER);
	if (tempStr) {
		strcpy(&mqtt_cfg.host[0],tempStr);
		free(tempStr);
	} else {
		ESP_LOGE(ME,"no server configured, terminating mqtt task");
		pvMqttTaskHandle = NULL;
		vTaskDelete(NULL);
	}
	// username (optional)
	tempStr  = configfile.get_str (MQ_CFG_USER);
	if (tempStr) {
		strcpy(&mqtt_cfg.username[0],tempStr);
		free(tempStr);
	} else {
		mqtt_cfg.username[0] = 0;
	}
	// password (optional)
	tempStr  = configfile.get_str (MQ_CFG_PASSWD);
	if (tempStr) {
		strcpy(&mqtt_cfg.password[0],tempStr);
		free(tempStr);
	} else {
		mqtt_cfg.password[0] = 0;
	}

	i = configfile.get_u32 (MQ_CFG_PORT);
	if (i == 0) i = MQ_TCP_DEF_PORT;
	mqtt_cfg.port = i;

	mqtt_cfg.event_handle = mqtt_event_handler;
    esp_mqtt_client_handle_t client = esp_mqtt_client_init(&mqtt_cfg);
    esp_mqtt_client_start(client);

    sendTimer = xTimerCreate(	"MQ-SEND",
								pdMS_TO_TICKS(mqtt_sendInterval),
								pdFALSE,       /* auto reload */
								(void *) 0,    /* some id */
								mqtt_timer_sendData_callback);
    if (sendTimer == NULL) { terminated++; ESP_LOGE(ME,"unable to create mqtt sendTimer"); }


	do {
		res = ev_receive (EV_TASK_MQTT, &event, &data, pdMS_TO_TICKS(sleepTime));
		if (res == EV_RC_OK) {
    		switch(event) {
    			case EV_MQTT_INTERVAL_CHANGE:
    				res = xTimerChangePeriod(sendTimer, data*1000, 500);
    				DMSG(ME, "timer change to %d, res: %d", data, res);
    				break;
    			case EV_END_TASK:
    				terminated = 1;
    				break;
    			case EV_MQTT_CONNECTED:
    				connected = 1;
    				msg_id = esp_mqtt_client_publish(client, "v1/devices/me/attributes", (const char*)&fw_ver, strlen(fw_ver), 1, 1);
    				DMSG(ME, "sent publish successful, msg_id=%d", msg_id);
    				if (xTimerStart (sendTimer, pdMS_TO_TICKS(1000)) != pdPASS) ESP_LOGE(ME,"unable to start mqtt sendTimer");
    				break;
    			case EV_MQTT_DISCONNECTED:
    				connected = 0;
    				if (xTimerStop (sendTimer, pdMS_TO_TICKS(10000)) != pdPASS) ESP_LOGE(ME,"unable to stop mqtt sendTimer");
    				break;
    			case EV_AVAILABLE_PWR_CHANGE:
    				v_powerFeedIntoGrid = data;
    				break;
    			case EV_CONSUMED_PWR_CHANGE:
    				//energyMeter_getPower3 (&v_pL1, &v_pL2, &v_pL3);
    				break;
    			case EV_CAR_POWER_PHASES_CHANGE:
    				v_phasesState = data;
    				break;
    			case EV_PWMAMPS_CHANGE:
    				v_pwmAmps = data;
    				break;
    			case EV_CABLE_CHANGE:
    				if (data > -1) v_cableAmps = data;
    				break;
    			case EV_MQTT_SEND_TIMER:
    				if (connected) {
    					snprintf(&varBuf[0],sizeof(varBuf),
    							MQV_S \
								  MQV_VARD("feedIntoGrid")  MQV_N \
								  MQV_VARD("chPwr")         MQV_N \
								  MQV_VARD("chPwrL1")       MQV_N \
								  MQV_VARD("chPwrL2")       MQV_N \
								  MQV_VARD("chPwrL3")       MQV_N \
								  MQV_VARF("UL1")           MQV_N \
								  MQV_VARF("UL2")           MQV_N \
								  MQV_VARF("UL3")           MQV_N \
								  MQV_VARF("IL1")           MQV_N \
								  MQV_VARF("IL2")           MQV_N \
								  MQV_VARF("IL3")           MQV_N \
								  MQV_VARD("L1")            MQV_N \
								  MQV_VARD("L2")            MQV_N \
								  MQV_VARD("L3")            MQV_N \
								  MQV_VARD("pwmA")          MQV_N \
								  MQV_VARD("cableA") \
								MQV_E,
								v_powerFeedIntoGrid,
								energyMeterData->pwrL1+energyMeterData->pwrL2+energyMeterData->pwrL3,energyMeterData->pwrL1,energyMeterData->pwrL2,energyMeterData->pwrL3,
								energyMeterData->voltageL1,energyMeterData->voltageL2,energyMeterData->voltageL3,
								energyMeterData->currL1,energyMeterData->currL2,energyMeterData->currL3,
								v_phasesState && 1, (v_phasesState && 2) >> 1, (v_phasesState && 4) >> 2,
								v_pwmAmps, v_cableAmps);
    					msg_id = esp_mqtt_client_publish(client, "v1/devices/me/telemetry", (const char*)&varBuf, strlen(varBuf), 1, 1);
    					DMSG(ME, "sent publish telemetry, msg_id=%d len:%d", msg_id, strlen(varBuf));
    				}
    		}
		}
		if (terminated == 0) {

        }
	} while (!(terminated));

	xTimerDelete (sendTimer, pdMS_TO_TICKS(2000));
    DMSG(ME, "calling mqtt_client_stop");
	esp_mqtt_client_stop(client);
	DMSG(ME, "calling mqtt_client_destroy");
	esp_mqtt_client_destroy(client);
	pvMqttTaskHandle = NULL;
    ESP_LOGI(TAG,"mqttTask terminated");
    vTaskDelete(NULL);  // without this -> crash
}



#define MQTT_RUNNING (pvMqttTaskHandle != NULL)


int mqtt_endTask(FILE *fd) {
	if (MQTT_RUNNING) {
		ev_send (EV_TASK_MQTT, EV_END_TASK, EV_DATA_NONE);
		int i = 0;
		while ((i<50) && (pvMqttTaskHandle != NULL)) {
			YIELD();
			i++;
		}
		if (pvMqttTaskHandle == NULL) {
			fprintf(fd,"mqtt task terminated\r\n");
			return 0;
		} else {
			fprintf(fd,"timeout waiting for mqtt task to terminate\r\n");
			return 1;
		}
	} else fprintf(fd, CONS_COLOR_STR_W_N("unable to stop mqttTask - task not started"));
	return 1;
}



int mqtt_startTask(FILE *fd) {
	char *server;

	if (pvMqttTaskHandle == NULL) {
		server = configfile.get_str (MQ_CFG_SERVER);
		if (server) {
			xTaskCreatePinnedToCore(mqttTask, "mqtt", 4096, NULL, EVSE_STD_TASK_PRIO, &pvMqttTaskHandle,APP_CPU);
			if (pvMqttTaskHandle == NULL) {
				fprintf(fd,CONS_COLOR_STR_E_N("\r\nxTaskCreate did not return a handle for the mqttTask"));
				return 1;
			}
		} else fprintf(fd,CONS_COLOR_STR_E_N("\rmqtt_startTask: no server configured"));
	} else
		fprintf(fd,CONS_COLOR_STR_E_N("\rmqttTask already running"));
	return 0;
}

/*******************************************************************************/


static struct {
	struct arg_lit *start    = arg_lit0("s", "start", "Start mqtt task");
	struct arg_lit *stop     = arg_lit0("t", "stop", "Stop mqtt task");
	struct arg_int *debug    = arg_int0("d","debug", NULL, "0=off, 1=debug messages on");
	struct arg_str *server   = arg_str0("r", "server", NULL, "servername or address (mqtt task will not be started if empty)");
	struct arg_str *username = arg_str0("u", "user", NULL, "optional: username (or access token) for mqtt login");
	struct arg_str *password = arg_str0("a", "pass", NULL, "optional: password for mqtt login");
	struct arg_int *port     = arg_int0("p", "port",NULL,"optional: server port (default 1883)");
	struct arg_lit *info     = arg_lit0("i", "info", "Show current state");
	struct arg_int *interval = arg_int0("n","interval", NULL, "set send interval in seconds");
    struct arg_end *end = arg_end(2);
} mqCommandArgs;


static int mq_command(FILE *fd, FILE *, int, char**) {
	int atLeastOneParam = 0;
	int saveNeeded = 0;
	int i;
	char *tempStr;
	uint32_t port;

	if (mqCommandArgs.server->count > 0) {
		configfile.set_str(MQ_CFG_SERVER,mqCommandArgs.server->sval[0]);
		saveNeeded++;
		fprintf(fd,"\rServer: %s\r\n",mqCommandArgs.server->sval[0]);
		atLeastOneParam++;
	}

	if (mqCommandArgs.username->count > 0) {
		configfile.set_str(MQ_CFG_USER,mqCommandArgs.username->sval[0]);
		saveNeeded++;
		fprintf(fd,"\rUser: %s\r\n",mqCommandArgs.username->sval[0]);
		atLeastOneParam++;
	}

	if (mqCommandArgs.password->count > 0) {
		configfile.set_str(MQ_CFG_PASSWD,mqCommandArgs.password->sval[0]);
		saveNeeded++;
		fprintf(fd,"\rPassword: %s\r\n",mqCommandArgs.password->sval[0]);
		atLeastOneParam++;
	}

	if (mqCommandArgs.port->count > 0) {
		configfile.set_u32(MQ_CFG_PORT,mqCommandArgs.port->ival[0]);
		saveNeeded++;
		fprintf(fd,"\rPort: %d\r\n",mqCommandArgs.port->ival[0]);
		atLeastOneParam++;
	}

	if (mqCommandArgs.interval->count > 0) {
		i = mqCommandArgs.interval->ival[0];
		if ((i*1000) < MQ_SEND_EVERY_MSECS_MIN) i = MQ_SEND_EVERY_MSECS_MIN / 1000;
		configfile.set_u32(MQ_CFG_INTERVAL,i);
		ev_send (EV_TASK_MQTT,EV_MQTT_INTERVAL_CHANGE,i);
		saveNeeded++; atLeastOneParam++;
	}

	if (saveNeeded) configfile.save();

	if (mqCommandArgs.debug->count > 0) {
		atLeastOneParam++;
		debug = mqCommandArgs.debug->ival[0];
		if (debug == 0) {
			if (debug_stderr != NULL) {
				//fclose(debug_stderr);		// close debug file stream - dont do this we need to dup but esp has no dup
				debug_stderr = NULL;
			}
		} else {
			//if (debug_stderr != NULL) fclose(debug_stderr);
			debug_stderr = fdopen (fileno(fd), "w");	// open it, we may be on another terminal
		}
		atLeastOneParam++;
		fprintf(fd,CONS_COLOR_STR_I_N("mqtt debug level: %d"),debug);
	}

	if ((mqCommandArgs.start->count) && (mqCommandArgs.stop->count)) {
		fprintf(fd,"what ? start or stop please\r\n");
		return 1;
	}
	if (mqCommandArgs.start->count) {
		mqtt_startTask(fd);
		atLeastOneParam++;
	}
	if (mqCommandArgs.stop->count) {
		mqtt_endTask(fd);
		atLeastOneParam++;
	}

	if (mqCommandArgs.info->count > 0) {
		if (! MQTT_RUNNING) {
			fprintf(fd, CONS_COLOR_STR_W_N("mqttTask not running"));
		}
		tempStr = configfile.get_str (MQ_CFG_SERVER);
		if (tempStr) {
			fprintf(fd,"\rServer: %s\r\n",tempStr); free(tempStr);
		} else {
			fprintf(fd,"\rServer: no server configured\r\n");
		}

		tempStr = configfile.get_str (MQ_CFG_USER);
		if (tempStr) {
			fprintf(fd,"\r  User: %s\r\n",tempStr); free(tempStr);
		}
		tempStr = configfile.get_str (MQ_CFG_PASSWD);
		if (tempStr) {
			fprintf(fd,"\rpassword set\r\n"); free(tempStr);
		} else fprintf(fd,"\rno password\r\n");

		port = configfile.get_u32 (MQ_CFG_PORT);
		fprintf(fd,"\r  Port: %u\r\n",port);

		atLeastOneParam++;
	}
	if (atLeastOneParam == 0) {
		arg_print_glossary(fd, (void**) &mqCommandArgs, NULL);
	}
	return 0;
}


void register_mqttCommands() {  /* register console commands */
	CMD_REG(mq, "start, stop or query mqtt task", NULL, &mq_command, &mqCommandArgs);
}
