/*
 * ESP32 (esp-idf)
 * generate the 1 KHz PWM for max current
 * A.Diehl 3. Nov. 2017
*/



#ifndef j1772pilot_h
#define j1772pilot_h

#include <time.h>

// use gpio ports or an mcp23017 for setting the relais
#define USE_PIO

#ifdef USE_PIO
#include "mcp23017_portio.h"
#endif

//#define j1772_PILOT_P12 0xffff
//#define j1772_PILOT_M12 0

#ifdef ESP_PLATFORM
#include "driver/adc.h"
#define ADC_CH_CABLEDETECT ADC1_CHANNEL_6
#define ADC_CH_PILOT       ADC1_CHANNEL_7
#else
#define ADC_CH_CABLEDETECT 6
#define ADC_CH_PILOT       7
#endif

// special value for amps
#define j1772_PILOT_P12 -1
#define j1772_PILOT_M12 -2
#define j1772_PILOT_OFF -3

#define PWM_12V() j1772pwm_setAmps(j1772_PILOT_P12,0)
#define PWM_MINUS12V() j1772pwm_setAmps(j1772_PILOT_M12,0)
#define PWM_OFF() j1772pwm_setAmps(j1772_PILOT_OFF,0)

#define j1772_PWM_AMPS_MIN 6
#define j1772_AMPS_MIN_SWITCH_TO_INC_PHASES 5
#define j1772_AMPS_MAX_DEFAULT 16
#define j1772_DETECT_MIN_WATTS_PHASE 99
#define j1772_DETECT_MIN_WATTS (j1772_DETECT_MIN_WATTS_PHASE*3)

#define j1772_pilot_GPIO (23)
#define j1772_pilotOff_GPIO		GPIO_NUM_15		/* to switch pilot signal off = 0 Volt */


#ifdef USE_PIO
// mcp23017 index
#define j1772_PIO_IDX 0

// bits for relais
// bits for keys
#define j1772_REL_L1_BIT 0
#define j1772_REL_L2_BIT 1
#define j1772_REL_L3_BIT 2

#else
#define j1772_L1_GPIO			GPIO_NUM_13		/* Relais L1,2,3 */
#define j1772_L2_GPIO			GPIO_NUM_2
#define j1772_L3_GPIO			GPIO_NUM_15
#endif

// delay for relays on/off #of ev_receive
#define j1772_RELAY_DELAY 40
#define j1772_PHASEDETECT_DELAY 40
#define j1772_CHANGE_PHASE_DELAY_MS 750

#define GRID_VOLTAGE 230

typedef enum {
    J1772_CABLE_DISCONNECTED = 0,
	J1772_CABLE_CONNECTED,
	J1772_CAR_READY,
	J1772_CAR_CHARGE,
	J1772_CAR_CHARGE_VENT,
	J1772_CAR_ERROR,
	J1772_LAST
} j1772_status_t;


int j1772_endTask(FILE *fd);
int j1772_startTask(FILE *fd);

void j1772pwm_portsInitialize();
void j1772pwm_initialize(FILE *fd);
void j1772pwm_setAmps(int Amps);
void register_j1772commands();
int j1772_carConnected();  // // returns 1 if car is connected
int j1772_carCharging();  // // returns 1 if car is charging

struct j1772_manual_settings_t {
	int		enabled;
	int     man_amps;		// 0 for max
	int     man_phases;		// 0 for auto, bit 0=L1, 1=L2, 2=L3
	time_t	startTime;		// -1 for now
	time_t  endTime;		// -1 for never end automatically
};

// caller needs to allocate/free the struct
void j1772_getManualSettings (j1772_manual_settings_t * settings);
void j1772_updateManualSettings (j1772_manual_settings_t * settings);
int j1772_manualModeEnabled ();
int j1772_getPwmAmps ();
int j1772_minAmps();
int j1772_maxAmps ();


#endif
