/*
 * queryInverter.cpp
 *
 * query Fronius Symo for available power
 *
 *  Created on: Dec 26, 2017
 *      Author: ad
 */
//#define test_linuxnative

#include "queryInverter.h"
#ifndef test_linuxnative
#include "evse_global.h"
#endif
#include "network.h"
#include "cons.h"
#include "argtable3_ad.h"

#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netdb.h>

#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <arpa/inet.h>
#include "cJSON.h"
#include <unistd.h>
#include <cJSON.h>
#include <stdarg.h>

#include <fcntl.h>

#ifndef ESP_LOGE
#define ESP_LOGE(TAG, args...) { fprintf(stderr,TAG); fprintf(stderr,": "); fprintf (stderr, args); fprintf(stderr,"\n"); }
#define ESP_LOGI(TAG, args...) { fprintf(stderr,TAG); fprintf(stderr,": "); fprintf (stderr, args); fprintf(stderr,"\n"); }
#endif

#ifndef UNUSED
#define UNUSED(x) x  __attribute__((unused))
#endif

#ifdef test_linuxnative
typedef int ev_event_t;
typedef int ev_data_t;
#endif

static const char* TAG = "queryInverter";

#define WEB_URL "http://fronius.armin.d/solar_api/GetAPIVersion.cgi"

static int debug = 0;
static FILE * debug_stderr = NULL;
static const char * MODULE = "inv   ";


void queryInverter_setDebug (FILE *fd, int level) {
	debug = level;
	if (level == 0) {
		if (debug_stderr != NULL) {
			fclose(debug_stderr);		// close debug file stream
			debug_stderr = NULL;
		}
	} else {
		if (debug_stderr != NULL) fclose(debug_stderr);
		debug_stderr = fdopen (fileno(fd), "w");	// open it, we may be on another terminal
	}
}

static void DMSG(
        const char* tag,
        const char* format, ...) __attribute__ ((format (printf, 2, 3)));

static void DMSG(
        const char* tag,
        const char* format, ...) {
	if (debug) {
		if (debug_stderr != NULL) {
			va_list list;
			va_start(list, format);
			fprintf(debug_stderr,CONS_COLOR_STR_I("\r%s"),MODULE);
			fprintf(debug_stderr,CONS_COLOR_STR_T("%-20s"), tag);
			//fprintf(debug_stderr,": ");
			vfprintf(debug_stderr,format, list);
			va_end(list);
			fprintf(debug_stderr,"\r\n");
			fflush(debug_stderr);
		}
		if (ferror(debug_stderr) != 0) {
			fclose(debug_stderr);
			debug_stderr = NULL;
			debug = 0;
			ESP_LOGE(TAG,"debug stdout no longer valid, debug mode disabled");
		}
	}
}

/*
static const char *REQUEST = "GET " WEB_URL " HTTP/1.0\r\n"
    "Host: "WEB_SERVER"\r\n"
    "User-Agent: esp-idf/1.0 esp32\r\n"
    "\r\n";
    */
//static const char *REQUEST =
////"GET /solar_api/GetAPIVersion.cgi HTTP/1.1\r\n"
//"Host: fronius.armin.d\r\n"
//"Connection: keep-alive\r\n\r\n";


//23960], length 367: HTTP, length: 367
//	GET /solar_api/GetAPIVersion.cgi HTTP/1.1
//	Host: fronius.armin.d
//	User-Agent: Mozilla/5.0 (X11; Fedora; Linux x86_64; rv:56.0) Gecko/20100101 Firefox/56.0
//	Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8
//	Accept-Language: en-US,en;q=0.5
//*	Accept-Encoding: gzip, deflate
//*	Cookie: lang=en
//	Connection: keep-alive
//*	Upgrade-Insecure-Requests: 1



#define MAXBUFLEN 2048

int socketConnect (const char *addrOrName, const char *port) {
	int sockfd,err;
	struct addrinfo hints;
    struct addrinfo *res;

    memset(&hints, 0, sizeof(hints));
    //hints.ai_family=AF_UNSPEC;
    hints.ai_family = AF_INET;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_protocol = IPPROTO_TCP;

	err = getaddrinfo(addrOrName, port, &hints, &res);

    if (err != 0 || res == NULL) {
        DMSG(TAG,"socketConnect: getaddrinfo for '%s'failed err=%d res=%p", addrOrName, err, res);
        return -1;
    }

    sockfd = socket(res->ai_family, res->ai_socktype, 0);
    YIELD();
    if (sockfd < 0) {
       DMSG(TAG,"socketConnect: Failed to allocate socket.");
       freeaddrinfo(res);
       return -1;
    }
/*
	int opts = fcntl(sockfd,F_GETFL);
    if (opts < 0) {
        ESP_LOGE(TAG,"fcntl(F_GETFL)");
    } else  {
		//opts = (opts | O_NONBLOCK);
		if (opts & O_NONBLOCK) {
			ESP_LOGI(TAG,"was set to nonblocking");
		} else {
			ESP_LOGI(TAG,"was set to blocking");
		}
	}
*/


    if (connect(sockfd, res->ai_addr, res->ai_addrlen) != 0) {
       DMSG(TAG,"socketConnect: socket connect failed errno=%d", errno);
       close(sockfd);
       freeaddrinfo(res);
       return -1;
    }

    freeaddrinfo(res);
    return sockfd;
}


int socketRequestReply (const char *host, const char *port, const char *requestPath, int maxlen, char *replyBuf) {
    int n,replyLen,requestLen,sockfd;
    char *s;
    char requestBuf[255];

    sockfd = socketConnect (host,port);
    if (sockfd < 0) return sockfd;
    YIELD();

    //sprintf(requestBuf,"GET %s HTTP/1.1\r\nHost: fronius.armin.d\r\nConnection: keep-alive\r\n\r\n", requestPath);
    sprintf(requestBuf,"GET %s HTTP/1.0\r\n\r\n", requestPath);
    requestLen = strlen(requestBuf);

    n = write(sockfd, requestBuf, requestLen);
    YIELD();

    if (n < requestLen) {
        DMSG(TAG,"socketRequestReply: socket send failed %d %d\n",n,requestLen);
        close(sockfd);
        return -1;
    }
    YIELD();
    replyLen=0;
    // need to use select to allow some time for the reply ?
    s=replyBuf;
    while ( (n = read(sockfd, s, maxlen-1)) > 0)
    {
		s+=n;
        *s = 0;
        replyLen+=n;
        YIELD();
    }
    //printf ("Reply:\n%s\n-------------------------------------\n",replyBuf);
    if (replyLen < 10)
		DMSG(TAG,"socketRequestReply: got only %d bytes ?\n",replyLen);
    close(sockfd);
    return replyLen;
}


#define PFXBUFLEN 256
int getFroniusWebApiPrefix (const char *host, const char *port, int maxlen, char *prefix) {
	char *buf;
	int n;
	char *jsonStart;

	buf = (char *)malloc(PFXBUFLEN);
	*buf = 0;
	if (buf == NULL) {
		DMSG(TAG,"getFroniusWebApiPrefix: no available memory for buffer");
		return -1;
	}

	*prefix = 0;
	n = socketRequestReply(host, port, "/solar_api/GetAPIVersion.cgi", PFXBUFLEN, buf);
	if (n < 0) return -1;

    jsonStart = strstr(buf,"\r\n\r\n");
    if (jsonStart == NULL) {
		DMSG	(TAG,"unable to find json start in response '%s'", buf);
		free(buf);
		return -1;
	}

    cJSON *root = cJSON_Parse(jsonStart);
	if (root == NULL) {
		DMSG(TAG,"Parse of GetApiVersion root failed");
		free(buf);
		return -1;
	}
	//printf("root type: %d\n",root->type);

	cJSON *url = cJSON_GetObjectItem(root,"BaseURL");
	if (url == NULL) {
		DMSG(TAG,"Parse of GetApiVersion url failed");
		cJSON_Delete(root);
		free(buf);
		return -1;
	}
	//printf("url type: %d\n",url->type);
	if (url->type != cJSON_String) {
		DMSG(TAG,"Parse of GetApiVersion url failed, string expected but got %d",url->type);
		cJSON_Delete(root);
		free(buf);
		return -1;
	}

	strncpy(prefix,url->valuestring,maxlen);
	//printf("Base URL: '%s'\n",prefix);

	cJSON_Delete(root);
	free(buf);

	return 1;
}



#define PWRBUFLEN 2048
int getAvailablePower (const char *host, const char *port, const char *prefix) {
	char *buf;
	int n;
	char *jsonStart;
	char path[256];

	buf = (char *) malloc(PWRBUFLEN);
	if (buf == NULL) {
		DMSG(TAG,"getAvailablePower: no available memory for buffer");
		return -1;
	}
	// http://fronius.armin.d/solar_api/v1/GetPowerFlowRealtimeData.fcgi
	sprintf(path,"%s/%s", prefix, "GetPowerFlowRealtimeData.fcgi");

	n = socketRequestReply(host, port, path, PWRBUFLEN, buf);
	if (n < 0) return -1;

    jsonStart = strstr(buf,"\r\n\r\n");
    if (jsonStart == NULL) {
		DMSG(TAG,"getAvailablePower: unable to find json start in response '%s'", buf);
		free(buf);
		return -1;
	}

    cJSON *root = cJSON_Parse(jsonStart);
	if (root == NULL) {
		DMSG(TAG,"getAvailablePower: Parse of root failed");
		free(buf);
		return -1;
	}
	//printf("root type: %d\n",root->type);

	cJSON *body = cJSON_GetObjectItem(root,"Body");
	if (body == NULL) {
		DMSG(TAG,"getAvailablePower: Parse of Body failed");
		cJSON_Delete(root);
		free(buf);
		return -1;
	}

	cJSON *data = cJSON_GetObjectItem(body,"Data");
	if (data == NULL) {
		DMSG(TAG,"getAvailablePower: Parse of Data failed");
		cJSON_Delete(root);
		free(buf);
		return -1;
	}

	cJSON *site = cJSON_GetObjectItem(data,"Site");
	if (site == NULL) {
		DMSG(TAG,"getAvailablePower: Parse of Site failed");
		cJSON_Delete(root);
		free(buf);
		return -1;
	}

	cJSON *P_Grid = cJSON_GetObjectItem(site,"P_Grid");
	if (P_Grid == NULL) {
		DMSG(TAG,"getAvailablePower: Parse of P_Grid failed");
		cJSON_Delete(root);
		free(buf);
		return -1;
	}

	if (P_Grid->type != cJSON_Number) {
		DMSG(TAG,"getAvailablePower: Parse of P_Grid failed, number expected but got %d",P_Grid->type);
		cJSON_Delete(root);
		free(buf);
		return -1;
	}

	n = P_Grid->valueint;

	//if (n > 0) n = 0; else n = n * -1;
	n = n * -1;	// we need negative values because the charger consumption needs to be separated later (in j1772)

	cJSON_Delete(root);
	free(buf);

	return n;
}



void queryInverter_setAvailablePower (int powerWatt) {
	//queue event for available power
    ev_send (EV_TASK_UI+EV_TASK_J1772+EV_TASK_MQTT, EV_AVAILABLE_PWR_CHANGE, powerWatt);
}


TaskHandle_t xQueryInverterTaskHandle = NULL;


extern "C" void queryInverterTask(void * UNUSED(param)) {
    int n = 0;
    char prefix[50];
    int havePrefix = 0;
    int res;
    ev_event_t event;
    ev_data_t data;
    int connected;
    int availablePowerLast = -2;

    char * host; //= "fronius.armin.d";
    const char * port = "80";

    host = configfile.get_str("INV_HOST");
    if (! host)
    	host = strdup (INV_DEFAULT_HOST);

    // in case the task has been stopped and restarted we need to check
    // if we already have a network connection
    connected = network_isConnected();

    while (1) {
#ifndef test_linuxnative
    	res = ev_receive (EV_TASK_INVERTER, &event, &data, pdMS_TO_TICKS(5000));  // TODO: make it configurable
    	if (res == EV_RC_OK) {
    		switch(event) {
    			case EV_END_TASK:
    				xQueryInverterTaskHandle = NULL;
    				DMSG(TAG,"ending task");
    				free(host);
    				vTaskDelete(NULL);
    				assert(0);
    				break;
    			case EV_NET_CONNECTED:
    				connected = 1;
    				DMSG(TAG,"network connected");
    				break;
    			case EV_NET_DISCONNECTED:
    				connected = 0;
    				havePrefix = 0;
    				availablePowerLast = -2;
    				DMSG(TAG,"network disconnected");
    				break;
    		}
    	}
#endif

    	if (connected) {
    		if (havePrefix == 0) {
    			n = getFroniusWebApiPrefix (host, port, sizeof(prefix)-1, prefix);
    			if (n < 0) {
    				DMSG(TAG,"unable to get solarweb api prefix");
    			} else {
    				//ESP_LOGI(TAG,"Prefix is '%s'\n",prefix);
    				havePrefix++;
    			}
    		}

    		if (havePrefix) {	// get power if we already have the prefix
    			n = getAvailablePower (host,port,prefix);
    			if (n != availablePowerLast) {
    				DMSG(TAG,"getAvailablePower returned %d Watt available @ inverter",n);
    				availablePowerLast = n;
#ifndef test_linuxnative
    				//queue event for available power
    				queryInverter_setAvailablePower (n);
#endif
    			}
    			if (n < 0) {  // there was an error reading the power, query the prefix the next time
    				havePrefix = 0;
    				availablePowerLast = -2;
    			}
    		}
    	} else {
    		DMSG(TAG,"sorry, no network connection(yet)");
    	}
    }
}


#ifdef test_linuxnative
int main () {
		queryInverterTask(NULL);
}	
#endif

#ifndef test_linuxnative


void queryInverter_initialize(void) {
	if (xQueryInverterTaskHandle == NULL) {
		xTaskCreatePinnedToCore(queryInverterTask, "queryInverter", 8192, NULL, EVSE_STD_TASK_PRIO, &xQueryInverterTaskHandle, APP_CPU);
		if (xQueryInverterTaskHandle == NULL) {
			ESP_LOGE(TAG,"xTaskCreate(queryInverter) failed");
		}
	}
	//ESP_LOGI(TAG,"task created");
}


void queryInverter_startTask(FILE *fd) {
	if (xQueryInverterTaskHandle == NULL) {
		queryInverter_initialize();
		if (xQueryInverterTaskHandle == NULL) fprintf(fd,CONS_COLOR_STR_E_N("unable to start task"));
	} else fprintf(fd,CONS_COLOR_STR_E_N("\rinverter task already running"));
}


int queryInverter_taskStarted() {
	return (xQueryInverterTaskHandle != NULL);
}


void queryInverter_endTask(FILE *fd) {
	if (xQueryInverterTaskHandle != NULL) {
		ev_send (EV_TASK_INVERTER, EV_END_TASK, EV_DATA_NONE);
		int i = 0;
		while ((i<50) && (xQueryInverterTaskHandle != NULL)) {
			YIELD();
			i++;
		}
		if (xQueryInverterTaskHandle == NULL) {
			fprintf(fd,"inverter task terminated\r\n");
		} else {
			fprintf(fd,"timeout waiting for inverter task to terminate\r\n");
		}
	} else fprintf(fd,"inverter task not started\r\n");
}

#endif


//****************************************************************************************

static struct {
	struct arg_lit *start = arg_lit0("s", "start", "Start query inverter task");
	struct arg_lit *stop  = arg_lit0("t", "stop", "Stop query inverter task");
	struct arg_str *server= arg_str0("r", "server", NULL, "servername or address");
	struct arg_int *debug = arg_int0("d","debug", NULL, "0=off, 1=query inverter debug messages on");
	struct arg_lit *minus = arg_lit0("m", "minus", "interpret <watt> as negative number");
	struct arg_int *pwr   = arg_int0 (NULL,NULL,"<watt>","set available power (valid only if query task not running)");
    struct arg_end *end = arg_end(2);
} inv_commandArgs;


static int inv_command(FILE *fd, FILE *, int, char**) {
	int atLeastOneParam = 0;
	int watt;

	if (inv_commandArgs.server->count > 0) {
		configfile.set_str("INV_HOST",inv_commandArgs.server->sval[0]);
		configfile.save();
		fprintf(fd,"\rServer: %s\r\n",inv_commandArgs.server->sval[0]);
		atLeastOneParam++;
	}

	if (inv_commandArgs.debug->count > 0) {
		int level = inv_commandArgs.debug->ival[0];
		queryInverter_setDebug (fd, level);
		fprintf(fd,CONS_COLOR_STR_I_N("inv debug level: %d"),level);
		atLeastOneParam++;
	}

	if ((inv_commandArgs.start->count) && (inv_commandArgs.stop->count)) {
		fprintf(fd,"what ? start or stop please\r\n");
		return 1;
	}
	if (inv_commandArgs.start->count) {
		queryInverter_startTask(fd);
		atLeastOneParam++;
	}
	if (inv_commandArgs.stop->count) {
		queryInverter_endTask(fd);
		atLeastOneParam++;
	}
	if (inv_commandArgs.pwr->count > 0) {
		watt = inv_commandArgs.pwr->ival[0];
		if (inv_commandArgs.minus->count > 0) watt = watt * -1;
		queryInverter_setAvailablePower(watt);
		atLeastOneParam++;
	}
	if (atLeastOneParam == 0) {
		arg_print_glossary(fd, (void**) &inv_commandArgs, NULL);
	}
	return 0;
}

void register_InvCommands() {
	CMD_REG2(inv		,"Start/Stop query inverter task or set available power (debug if task is stopped)", NULL);
}


